Package: ds4psy
Type: Package
Title: Data Science for Psychologists
Version: 1.3.0
Date: 2026-04-22
Authors@R: c(person("Hansjoerg", "Neth", role = c("aut", "cre"), email = "h.neth@uni.kn", comment = c(ORCID = "0000-0001-5427-3141")))
Maintainer: Hansjoerg Neth <h.neth@uni.kn>
Description: All datasets and functions required for the examples and exercises of the book "Data Science for Psychologists" (by Hansjoerg Neth, Konstanz University, 2026, <doi:10.5281/zenodo.7229812>), freely available at <https://hneth-ds4psy.share.connect.posit.cloud/>. The book and corresponding courses introduce principles and methods of data science to students of psychology and other biological or social sciences. The 'ds4psy' package primarily provides datasets, but also functions for data generation and manipulation (e.g., of text and time data) and graphics that are used in the book and its exercises. All functions included in 'ds4psy' are designed to be explicit and instructive, rather than efficient or elegant. 
Depends: R (>= 3.5.0)
Imports: ggplot2, unikn
Suggests: knitr, rmarkdown, spelling, testthat (>= 3.0.0)
Collate: 'util_fun.R' 'num_util_fun.R' 'text_util_fun.R'
        'time_util_fun.R' 'color_fun.R' 'data.R' 'data_fun.R'
        'text_fun.R' 'time_fun.R' 'num_fun.R' 'theme_fun.R'
        'plot_fun.R' 'start.R'
Encoding: UTF-8
LazyData: true
License: CC BY-SA 4.0
URL: https://hneth-ds4psy.share.connect.posit.cloud/,
        https://github.com/hneth/ds4psy/
BugReports: https://github.com/hneth/ds4psy/issues
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-22 06:26:09 UTC; hneth
Author: Hansjoerg Neth [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5427-3141>)
Repository: CRAN
Date/Publication: 2026-04-22 06:50:03 UTC
Built: R 4.5.2; ; 2026-04-22 09:42:55 UTC; unix
