# Generated by roxygen2: do not edit by hand

export(Umlaut)
export(base2dec)
export(base_digits)
export(capitalize)
export(caseflip)
export(cclass)
export(change_time)
export(change_tz)
export(chars_to_text)
export(coin)
export(collapse_chars)
export(count_chars)
export(count_chars_words)
export(count_words)
export(cur_date)
export(cur_time)
export(days_in_month)
export(dec2base)
export(dice)
export(dice_2)
export(diff_dates)
export(diff_times)
export(diff_tz)
export(ds4psy.guide)
export(get_set)
export(invert_rules)
export(is_equal)
export(is_leap_year)
export(is_vect)
export(is_wholenumber)
export(l33t_rul35)
export(make_grid)
export(map_text_coord)
export(map_text_regex)
export(metachar)
export(num_as_char)
export(num_as_ordinal)
export(num_equal)
export(pal_ds4psy)
export(pal_n_sq)
export(plot_charmap)
export(plot_chars)
export(plot_circ_points)
export(plot_fn)
export(plot_fun)
export(plot_n)
export(plot_text)
export(plot_tiles)
export(read_ascii)
export(sample_char)
export(sample_date)
export(sample_time)
export(text_to_chars)
export(text_to_sentences)
export(text_to_words)
export(theme_clean)
export(theme_ds4psy)
export(theme_empty)
export(transl33t)
export(what_date)
export(what_month)
export(what_time)
export(what_wday)
export(what_week)
export(what_year)
export(words_to_text)
export(zodiac)
import(datasets)
import(ggplot2)
import(grDevices)
import(unikn)
import(utils)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(stats,runif)
